@@ -1,6 +1,6 @@
 package JE::Object::Error::SyntaxError;
 
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 
 
 use strict;
@@ -47,10 +47,13 @@ sub new_constructor {
 		},
 		sub {
 			my $proto = shift;
-			my $global = $$proto->{global};
+			(my $global = $$proto->{global})->prototype_for(
+				SyntaxError => $proto
+			);
 			bless $proto, __PACKAGE__;
 			$proto->prototype(
-				$global->prop('Error')->prop('prototype')
+			   $global->prototype_for('Error')
+			|| $global->prop('Error')->prop('prototype')
 			);
 			$proto->prop({
 				name  => 'name',
